import java.awt.Color;
import java.util.ArrayList;
import java.util.TreeSet;

import javax.vecmath.Point2i;
import javax.vecmath.Vector2f;
import javax.vecmath.Vector3f;

import behaviors.ArrivalBehavior;
import behaviors.AvoidNeighboursBehavior;
import behaviors.AvoidObstaclesBehavior;
import behaviors.FleeBehavior;
import behaviors.PathFollowingBehavior;
import behaviors.SeekBehavior;
import behaviors.TargetSpeedBehavior;
import behaviors.WanderBehavior;

import common.Rectangle2f;

import opensteer.AbstractVehicle;
import opensteer.MyVehicle;
import opensteer.Obstacle;
import opensteer.PolylinePathway;
import opensteer.SimpleVehicle;
import opensteer.SphericalObstacle;
import view.GrassView;
import view.PathwayView;
import view.SphercialObstacleView;
import view.StatisticsView;
import view.TargetView;
import view.VehicleView;
import core.PaintBox2D;
import core.Simulation;
import core.Timer;


public class Main {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		//Init simulation
		Simulation simulation = new Simulation(0.1f, new Rectangle2f(new Vector2f(0,0), new Vector2f(1024, 768)));
		
		//init visualization
		PaintBox2D paintBox2D = new PaintBox2D(1024,768);
		simulation.addObserver(paintBox2D);
		
		//Make obstacles
		ArrayList<Obstacle> obstacles = new ArrayList<Obstacle>();
		
		SphericalObstacle obstacle = new SphericalObstacle(70, new Vector3f(800, 400, 0), 0);
		obstacles.add(obstacle);
		simulation.addSimulatedObject(obstacle);
		
		obstacle = new SphericalObstacle(30,new Vector3f(480, 430, 0), 0);
		obstacles.add(obstacle);
		simulation.addSimulatedObject(obstacle);
		
		obstacle = new SphericalObstacle(40,new Vector3f(300, 80, 0), 0);
		obstacles.add(obstacle);
		simulation.addSimulatedObject(obstacle);
		
		obstacle = new SphericalObstacle(30,new Vector3f(510, 290, 0), 0);
		obstacles.add(obstacle);
		simulation.addSimulatedObject(obstacle);
		
		obstacle = new SphericalObstacle(70,new Vector3f(250, 550, 0), 0);
		obstacles.add(obstacle);
		simulation.addSimulatedObject(obstacle);
        
        Vector3f target = new Vector3f(850,600,0);
        
        //Make road
        
        Vector3f[] points = {
        	
        	new Vector3f(100, 150, 0),
        	new Vector3f(400, 100, 0),
        	new Vector3f(550, 150, 0),
        	new Vector3f(560, 300, 0),
        	new Vector3f(450, 400, 0),
        	new Vector3f(400, 600, 0),
        	new Vector3f(450, 650, 0),
        	new Vector3f(650, 650, 0),
        	new Vector3f(850, 600, 0),
        	new Vector3f(950, 550, 0),
        	new Vector3f(950, 450, 0),
        	new Vector3f(900, 350, 0),
        	new Vector3f(800, 250, 0),
        	new Vector3f(700, 200, 0),
        	new Vector3f(500, 200, 0),
        	new Vector3f(300, 250, 0),
        };
        
        PolylinePathway pathway = new PolylinePathway(points.length, points, 40, false);
		
		//Make vehicles
		ArrayList<SimpleVehicle> vehicles = new ArrayList<SimpleVehicle>();
		
//		for ( int i = 0; i < 3; i++) {
//			
//			MyVehicle vehicle = new MyVehicle();
//			vehicle.setPosition((float)Math.random()*1000,(float)Math.random()*700,0);
//			
//			vehicles.add(vehicle);
//			simulation.addSimulatedObject(vehicle);
//			
//			vehicle.addBehavior(new AvoidObstaclesBehavior(obstacles));
//	        vehicle.addBehavior(new AvoidNeighboursBehavior(vehicles,5));
//			vehicle.addBehavior(new TargetSpeedBehavior(vehicle.maxSpeed()));
//		}
		
		MyVehicle vehicle1 = new MyVehicle();
		vehicle1.setPosition(90,155,0);
		
		MyVehicle vehicle2 = new MyVehicle();
		vehicle2.setPosition(900,600,0);
		
		vehicles.add(vehicle1);
		vehicles.add(vehicle2);
		simulation.addSimulatedObject(vehicle1);
		simulation.addSimulatedObject(vehicle2);
		
		//Fill vehicles mind;
        
        
        //vehicle1.addBehavior(new AvoidNeighboursBehavior(vehicles,5));
		vehicle1.addBehavior(new AvoidObstaclesBehavior(obstacles));
		//vehicle1.addBehavior(new WanderBehavior(20));
		//vehicle1.addBehavior(new PathFollowingBehavior(pathway, 5));
		//vehicle1.addBehavior(new TargetSpeedBehavior(vehicle1.maxSpeed()));
		//vehicle1.addBehavior(new ArrivalBehavior(target, 300, 20));
		vehicle1.addBehavior(new SeekBehavior(vehicle2.position()));
		

        //vehicle2.addBehavior(new AvoidNeighboursBehavior(vehicles,5));
		vehicle2.addBehavior(new AvoidObstaclesBehavior(obstacles));
		//vehicle2.addBehavior(new WanderBehavior(5));
		//vehicle2.addBehavior(new PathFollowingBehavior(pathway, 5));
		vehicle2.addBehavior(new FleeBehavior(vehicle1.position()));
		vehicle2.addBehavior(new TargetSpeedBehavior(vehicle1.maxSpeed()));
		//vehicle2.addBehavior(new ArrivalBehavior(target, 300, 20));
		
		GrassView grassView = new GrassView(1024, 768);
		paintBox2D.addToCanvas(grassView);
		
		PathwayView pathwayView = new PathwayView(pathway);
		paintBox2D.addToCanvas(pathwayView);
		
//		Add obstacles
		TargetView targetView = new TargetView(target);
		paintBox2D.addToCanvas(targetView);
		
		for (Obstacle o : obstacles) {
			
			SphercialObstacleView obstacleView = new SphercialObstacleView((SphericalObstacle)o);
			paintBox2D.addToCanvas(obstacleView);
		}
		
		for (SimpleVehicle vehicle : vehicles) {
	        
			VehicleView vehicleView = 
				new VehicleView(vehicle, new Color((float)Math.random(), (float)Math.random(), (float)Math.random()));
			paintBox2D.addToCanvas(vehicleView);
        }
		
		//Add text objects
		StatisticsView vehicleStat = new StatisticsView(vehicle1, new Point2i(10, 640), Color.YELLOW);
		paintBox2D.addToCanvas(vehicleStat);
		
		vehicleStat = new StatisticsView(vehicle2, new Point2i(10, 570), Color.YELLOW);
		paintBox2D.addToCanvas(vehicleStat);
		
		StatisticsView simStat = new StatisticsView(simulation, new Point2i(10, 700), Color.WHITE);
		paintBox2D.addToCanvas(simStat);
		
		paintBox2D.render();
		
		Timer simulationTimer = new Timer(simulation,10);
		simulationTimer.start();
		simulationTimer.play();
	}

}
