package core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;

import javax.vecmath.Vector3f;

import com.sun.java_cup.internal.version;

import opensteer.MyVehicle;
import opensteer.SimpleVehicle;

import common.SimulatedObject;
import common.Rectangle2f;
import common.Statistics;
import common.StatisticsSource;
import common.Updatable;

public class Simulation extends Observable implements StatisticsSource, Updatable {
	
	private long simulationTime = 0;
	private float timeScale; 
    
    private ArrayList simutatedObjects = new ArrayList();
	
	private Rectangle2f bounds;
	private Statistics statistics = new Statistics(3);

	public Simulation(float timeScale, Rectangle2f bounds) {
		super();
		
		this.timeScale = timeScale;
		this.bounds = bounds;
		
		this.statistics.setNameValue(0,"Simulation","");
		this.statistics.setName(1,"Time");
		this.statistics.setNameValue(2,"Time scale", timeScale);
	}
	
	public float getSimulationTime() {
		
		return this.simulationTime * timeScale;
	}
	
	public boolean computeRebound(SimpleVehicle vehicle) {
		
		for (Iterator i = this.simutatedObjects.iterator(); i.hasNext();) {
	        SimulatedObject object = (SimulatedObject) i.next();
	        
	        if ( object != vehicle) {

	        	Vector3f offset = new Vector3f(vehicle.position());
	        	offset.sub(object.position());

	        	float distance = offset.length();

	        	if ( distance < vehicle.radius() + object.radius() ) {

	        		Vector3f I = new Vector3f(vehicle.forward());

	        		I.negate();
	        		offset.normalize();

	        		float dot = offset.dot(I);
	        		
	        		if ( dot > 0 ) { 
	        			offset.scale( dot * 2 );

	        			vehicle.forward().add(offset);
	        			vehicle.setUnitSideFromForwardAndUp();

	        			return true;
	        		}
	        	}
	        }
        }
		return false;
	}
	
	public boolean correctPosition(MyVehicle vehicle) {
		
		Vector3f position = vehicle.position();
		
		if ( computeRebound(vehicle) ) {
			
//			this.deleteObserver(vehicle);
//			this.simutatedObjects.remove(vehicle);
		}
		
		boolean doCorrect = false;
		
		if ( position.x < bounds.getBottomLeftCorner().x ) {
			
			position.x += bounds.getWidth();
			doCorrect = true;
		}
		if ( position.x > bounds.getTopRightCorner().x ) {
			
			position.x -= bounds.getWidth();
			doCorrect = true;
		}
		if ( position.y < bounds.getBottomLeftCorner().y ) {
			
			position.y += bounds.getHeight();
			doCorrect = true;
		}
		if ( position.y > bounds.getTopRightCorner().y ) {
			
			position.y -= bounds.getHeight();
			doCorrect = true;
		}
		return doCorrect;
	}

	public Statistics getStatistics() {
		
		this.statistics.setValue(1,this.getSimulationTime());
		
		return statistics;
	}

	public void update() {
		
		this.setChanged();
		this.notifyObservers();
		
		simulationTime++;
	}
    
    public void addSimulatedObject(SimulatedObject object) {
     
        if ( object instanceof MyVehicle ) {
            
            this.addObserver((MyVehicle)object);
        }
        
        this.simutatedObjects.add(object);
    }
}
