package opensteer;

import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import javax.vecmath.Vector3f;

import behaviors.SteeringBehavior;

import common.Statistics;
import common.StatisticsSource;

import core.Simulation;

public class MyVehicle extends SteerVehicle implements Observer, StatisticsSource {
	
	private float lastSimulationTime = -1;
	private Statistics statistics = new Statistics(5);
	
	private ArrayList<SteeringBehavior> behaviors = new ArrayList<SteeringBehavior>();

	public MyVehicle() {
		super();
		
		this.setMass(5);
		this.setMaxForce(50);
		this.setMaxSpeed(30);
		
		this.setRadius(17);
		this.setSpeed((float)Math.random() * maxSpeed());
		
		this.forward.x = (float)Math.random();
		this.forward.y = (float)Math.random();
		this.forward().normalize();
		
		this.setUnitSideFromForwardAndUp();
		
		this.statistics.setNameValue(0, "Name", "Car");
		this.statistics.setNameValue(1, "Position", this.position);
		this.statistics.setNameValue(2, "Direction", this.forward);
		this.statistics.setNameValue(3, "Acceleration", this.smoothedAcceleration);
		this.statistics.setName(4, "Speed");
	}
	
	public void addBehavior(SteeringBehavior behavior) {
		
		this.behaviors.add(behavior);
	}

	public void update(Observable o, Object arg) {

		Simulation simulation = (Simulation)o;
		float time = simulation.getSimulationTime();
		
		if ( lastSimulationTime != -1 ) {
			
			float deltaTime = time - lastSimulationTime;
			Vector3f steerForce = new Vector3f(ZERO);
			
			for (SteeringBehavior behavior : this.behaviors) {
				
				boolean doCorrection = behavior.steer(this, steerForce);
				
				if ( doCorrection ) break;
			}
			
			this.applySteeringForce(steerForce, deltaTime);
			this.applyBrakingForce(0.1f, deltaTime);
			
			simulation.correctPosition(this);
		}
		this.lastSimulationTime = time;
	}

	public Statistics getStatistics() {
		
		this.statistics.setValue(4, speed());
		return this.statistics;
	}
}
