package opensteer;

import javax.vecmath.Vector3f;

// Converted from the original C++ by JIST
// Use this as a basis for transcribing into Java... (good luck!)




//----------------------------------------------------------------------------
//
//
//OpenSteer -- Steering Behaviors for Autonomous Characters
//
//Copyright (c) 2002-2003, Sony Computer Entertainment America
//Original author: Craig Reynolds <craig_reynolds@playstation.sony.com>
//
//Permission is hereby granted, free of charge, to any person obtaining a
//copy of this software and associated documentation files (the "Software"),
//to deal in the Software without restriction, including without limitation
//the rights to use, copy, modify, merge, publish, distribute, sublicense,
//and/or sell copies of the Software, and to permit persons to whom the
//Software is furnished to do so, subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
//THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//DEALINGS IN THE SOFTWARE.
//
//
//----------------------------------------------------------------------------
//
//
//Obstacle and SphericalObstacle
//
//for use with obstacle avoidance
//
//09-05-02 cwr: created
//
//
//----------------------------------------------------------------------------
//
//XXX 4-23-03: Temporary work around.  Put steerToAvoid method here to
//XXX support generic steerToAvoidObstacle method in SteerLibraryMixin.
//XXX That in turn requires making Obstacles know about AbstractVehicle.
//XXX The final implementation will probably be significantly different.
//
//----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// SphericalObstacle a simple concrete type of obstacle



public class SphericalObstacle implements Obstacle
{
    public float radius;
    public Vector3f center;
    private int seenFrom;

    public int seenFrom () {return seenFrom;}

    public void setSeenFrom (int s) {seenFrom = s;}



    // XXX 4-23-03: Temporary work around (see comment above)
    //
    // Checks for intersection of the given spherical obstacle with a
    // volume of "likely future vehicle positions": a cylinder along the
    // current path, extending minTimeToCollision seconds along the
    // forward axis from current position.
    //
    // If they intersect, a collision is imminent and this function returns
    // a steering force pointing laterally away from the obstacle's center.
    //
    // Returns a zero vector if the obstacle is outside the cylinder
    //
    // xxx couldn't this be made more compact using localizePosition?

    public Vector3f steerToAvoid (AbstractVehicle v, float minTimeToCollision)
    {
        // minimum distance to obstacle before avoidance is required
        final float minDistanceToCollision = minTimeToCollision * v.speed();
        final float minDistanceToCenter = minDistanceToCollision + radius;

        // contact distance: sum of radii of obstacle and vehicle
        final float totalRadius = radius + v.radius ();

        // obstacle center relative to vehicle position
        // final Vector3f localOffset = center - v.position ();
        final Vector3f localOffset = new Vector3f (center); localOffset.sub (v.position ());

        // distance along vehicle's forward axis to obstacle's center
        final float forwardComponent = localOffset.dot (v.forward ());
        // final Vector3f forwardOffset = forwardComponent * v.forward ();
        final Vector3f forwardOffset = new Vector3f (v.forward()); forwardOffset.scale (forwardComponent);

        // offset from forward axis to obstacle's center
        // final Vector3f offForwardOffset = localOffset - forwardOffset;
        final Vector3f offForwardOffset = new Vector3f (localOffset); offForwardOffset.sub (forwardOffset);  

        // test to see if sphere overlaps with obstacle-free corridor
        final boolean inCylinder = offForwardOffset.length() < totalRadius;
        final boolean nearby = forwardComponent < minDistanceToCenter;
        final boolean inFront = forwardComponent > 0;

        // if all three conditions are met, steer away from sphere center
        if (inCylinder && nearby && inFront)
        {
            // return offForwardOffset * -1;
            offForwardOffset.scale (-1);
            return offForwardOffset;
        }
        else
        {
            return LocalSpace.ZERO;
        }
    }

	public SphericalObstacle(float radius, Vector3f center, int seenFrom) {
		super();
		// TODO Auto-generated constructor stub
		this.radius = radius;
		this.center = center;
		this.seenFrom = seenFrom;
	}

    public Vector3f position() {
        
       return center;
    }
    
    public float radius() {
        
        return radius;
     }
}

