/*
 * Created on 31-Jan-2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package opensteer;

import javax.vecmath.Vector3f;

/**
 * @author Dave
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class Vector3fUtil
{
    public static float distance (Vector3f a, Vector3f b)
    {
        Vector3f v = new Vector3f (a); v.sub (b); return v.length();
    }
    
    // return component of vector parallel to a unit basis vector
    // (IMPORTANT NOTE: assumes "basis" has unit magnitude (length==1))

    public static Vector3f parallelComponent (Vector3f v, Vector3f unitBasis)
    {
        final float projection = v.dot (unitBasis);
        Vector3f w = new Vector3f (unitBasis); w.scale (projection);
        return w;
    }


    // return component of vector perpendicular to a unit basis vector
    // (IMPORTANT NOTE: assumes "basis" has unit magnitude (length==1))

    public static Vector3f perpendicularComponent (Vector3f v, Vector3f unitBasis)
    {
        Vector3f w = new Vector3f (v); w.sub (parallelComponent (v, unitBasis));
        return w;
    }


    // clamps the length of a given vector to maxLength.  If the vector is
    // shorter its value is returned unaltered, if the vector is longer
    // the value returned has length of maxLength and is paralle to the
    // original input.

    public static Vector3f truncateLength (Vector3f v, float maxLength)
    {
        final float maxLengthSquared = maxLength * maxLength;
        final float vecLengthSquared = v.lengthSquared ();
        if (vecLengthSquared <= maxLengthSquared)
        {
            return v;
        }
        else
        {
            Vector3f w = new Vector3f (v); 
            w.scale (maxLength / (float) Math.sqrt (vecLengthSquared));
            return w;
        }
    }

    //  ----------------------------------------------------------------------------
    //  Returns a position randomly distributed on the surface of a sphere
    //  of unit radius centered at the origin.  Orientation will be random
    //  and length will be 1

    public static Vector3f randomUnitVector()
    {
        Vector3f v = randomVectorInUnitRadiusSphere(); v.normalize(); return v;
    }

    //  ----------------------------------------------------------------------------
    //  Returns a position randomly distributed on a circle of unit radius
    //  on the XZ (Y=0) plane, centered at the origin.  Orientation will be
    //  random and length will be 1

    public static Vector3f randomUnitVectorOnXZPlane()
    {
        Vector3f v = randomVectorInUnitRadiusSphere(); v.y = 0; v.normalize(); return v; 
    }

    //  Returns a position randomly distributed inside a sphere of unit radius
    //  centered at the origin.  Orientation will be random and length will range
    //  between 0 and 1

    public static Vector3f randomVectorInUnitRadiusSphere()
    {
        Vector3f v = new Vector3f();

        do
        {
            v.set((float) (Math.random() * 2) - 1, (float) (Math.random() * 2) - 1, (float) (Math.random() * 2) - 1);
        }
        while (v.length() >= 1);

        return v;
    }

    //  ----------------------------------------------------------------------------
    //  Returns a position randomly distributed on a disk of unit radius
    //  on the XZ (Y=0) plane, centered at the origin.  Orientation will be
    //  random and length will range between 0 and 1

    public static Vector3f randomVectorOnUnitRadiusXZDisk()
    {
        Vector3f v = new Vector3f();

        do
        {
            v.set((float) (Math.random() * 2) - 1, 0, (float) (Math.random() * 2) - 1);
        }
        while (v.length() >= 1);

        return v;
    }

    //  ----------------------------------------------------------------------------
    //  Returns a position randomly distributed on a disk of unit radius
    //  on the XY (Z=0) plane, centered at the origin.  Orientation will be
    //  random and length will range between 0 and 1

    public static Vector3f randomVectorOnUnitRadiusXYDisk()
    {
        Vector3f v = new Vector3f();

        do
        {
            v.set((float) (Math.random() * 2) - 1, (float) (Math.random() * 2) - 1, 0);
        }
        while (v.length() >= 1);

        return v;
    }

    //  ----------------------------------------------------------------------------
    //  Does a "ceiling" or "floor" operation on the angle by which a given vector
    //  deviates from a given reference basis vector.  Consider a cone with "basis"
    //  as its axis and slope of "cosineOfConeAngle".  The first argument controls
    //  whether the "source" vector is forced to remain inside or outside of this
    //  cone.  Called by vecLimitMaxDeviationAngle and vecLimitMinDeviationAngle.

    public static Vector3f vecLimitDeviationAngleUtility(boolean insideOrOutside, Vector3f source, float cosineOfConeAngle, Vector3f basis)
    {
        // immediately return zero length input vectors
        float sourceLength = source.length();
        if (sourceLength == 0)
            return source;

        // measure the angular diviation of "source" from "basis"
        // final Vector3f direction = source / sourceLength;
        final Vector3f direction = new Vector3f (source); direction.scale (1 / sourceLength);
        float cosineOfSourceAngle = direction.dot(basis);

        // Simply return "source" if it already meets the angle criteria.
        // (note: we hope this top "if" gets compiled out since the flag
        // is a constant when the function is inlined into its caller)
        if (insideOrOutside)
        {
            // source vector is already inside the cone, just return it
            if (cosineOfSourceAngle >= cosineOfConeAngle)
                return source;
        }
        else
        {
            // source vector is already outside the cone, just return it
            if (cosineOfSourceAngle <= cosineOfConeAngle)
                return source;
        }

        // find the portion of "source" that is perpendicular to "basis"
        final Vector3f perp = Vector3fUtil.perpendicularComponent(source, basis);

        // normalize that perpendicular
        perp.normalize();
        final Vector3f unitPerp = perp;

        // construct a new vector whose length equals the source vector,
        // and lies on the intersection of a plane (formed the source and
        // basis vectors) and a cone (whose axis is "basis" and whose
        // angle corresponds to cosineOfConeAngle)
        float perpDist = (float) Math.sqrt(1 - (cosineOfConeAngle * cosineOfConeAngle));
        // final Vector3f c0 = basis * cosineOfConeAngle;
        final Vector3f c0 = new Vector3f (basis); c0.scale (cosineOfConeAngle); 
        // final Vector3f c1 = unitPerp * perpDist;
        final Vector3f c1 = new Vector3f (unitPerp); c1.scale (perpDist);
        // return (c0 + c1) * sourceLength;
        c0.add (c1); c0.scale (sourceLength); return c0;
    }

    //  ----------------------------------------------------------------------------
    //  Enforce an upper bound on the angle by which a given arbitrary vector
    //  diviates from a given reference direction (specified by a unit basis
    //  vector).  The effect is to clip the "source" vector to be inside a cone
    //  defined by the basis and an angle.
    
    public static Vector3f limitMaxDeviationAngle(Vector3f source, float cosineOfConeAngle, Vector3f basis)
    {
        return vecLimitDeviationAngleUtility(true, // force source INSIDE cone
                                             source, cosineOfConeAngle, basis);
    }
    
    //  ----------------------------------------------------------------------------
    //  Enforce a lower bound on the angle by which a given arbitrary vector
    //  diviates from a given reference direction (specified by a unit basis
    //  vector).  The effect is to clip the "source" vector to be outside a cone
    //  defined by the basis and an angle.
    
    public static Vector3f limitMinDeviationAngle(Vector3f source, float cosineOfConeAngle, Vector3f basis)
    {
        return vecLimitDeviationAngleUtility(false, // force source OUTSIDE cone
                                             source, cosineOfConeAngle, basis);
    }

    //  ----------------------------------------------------------------------------
    //  given a vector, return a vector perpendicular to it.  arbitrarily selects
    //  one of the infinitely many perpendicular vectors.  a zero vector maps to
    //  itself, otherwise length is irrelevant (empirically, output length seems to
    //  remain within 20% of input length).

    public static Vector3f findPerpendicularIn3d(Vector3f direction)
    {
        // to be filled in:
        Vector3f quasiPerp; // a direction which is "almost perpendicular"
        Vector3f result = new Vector3f(); // the computed perpendicular to be returned

        // three mutually perpendicular basis vectors
        final Vector3f i = new Vector3f(1, 0, 0);
        final Vector3f j = new Vector3f(0, 1, 0);
        final Vector3f k = new Vector3f(0, 0, 1);

        // measure the projection of "direction" onto each of the axes
        final float id = i.dot(direction);
        final float jd = j.dot(direction);
        final float kd = k.dot(direction);

        // set quasiPerp to the basis which is least parallel to "direction"
        if ((id <= jd) && (id <= kd))
        {
            quasiPerp = i; // projection onto i was the smallest
        }
        else
        {
            if ((jd <= id) && (jd <= kd))
                quasiPerp = j; // projection onto j was the smallest
            else
                quasiPerp = k; // projection onto k was the smallest
        }

        // return the cross product (direction x quasiPerp)
        // which is guaranteed to be perpendicular to both of them
        result.cross(direction, quasiPerp);
        return result;
    }

}
