/*
 * Created on 16.12.2006
 *
 */
package view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.TexturePaint;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.vecmath.Matrix3f;
import javax.vecmath.Vector3f;

import opensteer.PolylinePathway;

import common.DirectDrawable;
import common.Drawable;
import core.Util;

public class PathwayView implements DirectDrawable {
	
	private PolylinePathway pathway;
	private TexturePaint paint;
	private GeneralPath polyline;
	
	private BufferedImage image;


	/**
     * @param pathway
     */
    public PathwayView(PolylinePathway pathway) {
	    this.pathway = pathway;
		
		try {
		    image = ImageIO.read(new File("road.bmp"));
		} 
		catch (IOException e) {

		}
		
		this.polyline = 
			new GeneralPath(GeneralPath.WIND_EVEN_ODD, pathway.points.length);
			
		paint = new TexturePaint(image, new Rectangle2D.Double(0, 0, image.getWidth(), image.getHeight()));
    }

	public void drawDirect(Graphics2D g2, Matrix3f transform) {
	    
		Vector3f[] points = pathway.getPoints();	
	
		Vector3f tPosition = new Vector3f(points[0].x, points[0].y, 1);
		transform.transform(tPosition);

		polyline.moveTo(tPosition.x,tPosition.y);

		for (int i = 0; i < points.length; i++) {

			tPosition = new Vector3f(points[i].x, points[i].y, 1);
			transform.transform(tPosition);

			polyline.lineTo(tPosition.x, tPosition.y);
		}
		
		BasicStroke stroke = new BasicStroke(pathway.getRadius() * 2 + 5, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		Shape thickPolyLine = stroke.createStrokedShape(polyline);
		
		g2.setColor(Color.WHITE);
		g2.fill(thickPolyLine);
		
		stroke = new BasicStroke(pathway.getRadius() * 2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		thickPolyLine = stroke.createStrokedShape(polyline);
		
		Paint oldPaint = g2.getPaint();
		g2.setPaint(paint);

		g2.fill(thickPolyLine);
		
		g2.setPaint(oldPaint);
    }

}
