package view;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Locale;

import javax.vecmath.Matrix3f;
import javax.vecmath.Point2i;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

import common.Drawable;
import common.Statistics;
import common.StatisticsSource;
import core.Util;

public class StatisticsView implements Drawable {
	
	private Point2i position;
	private StatisticsSource statisticsSource;
	private Color color;
	private NumberFormat numberFormat = NumberFormat.getInstance(Locale.UK);

	public StatisticsView(StatisticsSource statisticsSource, Point2i position, Color color) {
		
		this.position = position;
		this.statisticsSource = statisticsSource;
		this.color = color;
		((DecimalFormat)this.numberFormat).applyPattern(" 000.000;-000.000");
	}

	public void draw(Graphics g, Matrix3f transform) {
		
		Statistics statistics = this.statisticsSource.getStatistics();
		Vector3f tPosition = new Vector3f(position.x, position.y, 1);
		
		transform.transform(tPosition);
		
		int x = Util.round(tPosition.x);
		int y = Util.round(tPosition.y);
		
		g.setColor(color);
		g.setFont(Font.getFont("Terminal"));
		
		for (int i = 0; i < statistics.getNames().length; i++) {
			
			Object value = statistics.getValue(i);
			
			if (value instanceof Float) {
			
				value = numberFormat.format(value);
			}
			else if ( value instanceof Tuple3f ) {
				
				Tuple3f vector = (Tuple3f)value;
				value = "( " + numberFormat.format(vector.x) + ", " + 
						numberFormat.format(vector.y) + ", " + numberFormat.format(vector.z) + " )";
			}
			
			g.drawString(statistics.getName(i) + ": " + value, x, y + 12*i);
		}
	}

}
