package view;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.vecmath.Matrix3f;
import javax.vecmath.Vector3f;

import common.Drawable;
import core.Util;

import opensteer.LocalSpace;
import opensteer.SimpleVehicle;

public class VehicleView implements Drawable {
	
	private Color color;
	private SimpleVehicle vehicle;
	private BufferedImage image;
	private BufferedImage maskImage;

	public VehicleView(SimpleVehicle vehicle, Color color) {
		super();
		// TODO Auto-generated constructor stub
		this.vehicle = vehicle;
		this.color = color;
		
		try {
			
		    image = ImageIO.read(new File("car1.png"));
		    maskImage = ImageIO.read(new File("car_mask.png"));
		} 
		catch (IOException e) {

		}
		
		Graphics2D g2 = maskImage.createGraphics();
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_IN, 1));
		g2.setColor(color);
		g2.fillRect(0, 0, image.getWidth(), image.getHeight());
		
		g2 = image.createGraphics();
		g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.7f));
		g2.drawImage(maskImage, 0, 0, null);
	}

	public void draw(Graphics g, Matrix3f transform) {
		
		Vector3f position = new Vector3f(this.vehicle.position().x, this.vehicle.position().y, 0);
		Vector3f forward = new Vector3f(this.vehicle.forward().x, this.vehicle.forward().y, 0);
		Vector3f acc = new Vector3f(this.vehicle.getSmoothedAcceleration().x, this.vehicle.getSmoothedAcceleration().y, 0);
		///int radius = Util.round(this.vehicle.radius());

		float dot = forward.dot(new Vector3f(0,1,0));
		float angle = (float)Math.acos(dot);
		
		if ( forward.x < 0 ) angle = -angle;
		
		forward.scale(30);
		forward.add(position);
		
		acc.scale(10);
		acc.add(position);
		
		position.z = 1;
		forward.z = 1;
		acc.z = 1; 

		transform.transform(position);
		transform.transform(forward);
		transform.transform(acc);

		int x = Util.round(position.x / position.z);
		int y = Util.round(position.y / position.z);
		
		int fx = Util.round(forward.x / forward.z);
		int fy = Util.round(forward.y / forward.z);
		
		int ax = Util.round(acc.x / acc.z);
		int ay = Util.round(acc.y / acc.z);
		
		int imgX = x + image.getWidth() / 2;
		int imgY = y - image.getHeight() / 2;
		
		//g.setColor(Color.WHITE);
		//g.fillOval(x - 12, y - 12, 24, 24);
		
		//g.setColor(color);
		//g.fillOval(x - 9, y - 9, 18, 18);
		
		Graphics2D g2 = (Graphics2D)g;
		
		AffineTransform carT = AffineTransform.getTranslateInstance(x, y);
		carT.concatenate(AffineTransform.getRotateInstance(angle));
		carT.concatenate(AffineTransform.getScaleInstance(1, 1));
		carT.concatenate(AffineTransform.getTranslateInstance(- image.getWidth() / 2, -image.getHeight() / 2));
		
		g2.drawImage(image, carT, null);
		
		//g.setColor(Color.RED);
		//g.drawLine(x, y, fx, fy);
		
		g.setColor(Color.BLUE);
		g.drawLine(x, y, ax, ay);
	}
}
